import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
/**
 * Copyright IBM Corp. 2020, 2024
 *
 * This source code is licensed under the Apache-2.0 license found in the
 * LICENSE file in the root directory of this source tree.
 */

import axios from 'axios';
import { DDOAPI } from '../DDO';
import ipcinfoCookie from '../../internal/vendor/@carbon/ibmdotcom-utilities/utilities/ipcinfoCookie/ipcinfoCookie';
import root from 'window-or-global';

/**
 * @typedef {object} Locale
 * @property {string} cc The country code.
 * @property {string} lc The language code.
 */

/**
 * @constant {string | string} Host for the Locale API call.
 * @private
 */
var _host = process && (process.env.REACT_APP_TRANSLATION_HOST || process.env.TRANSLATION_HOST) || 'https://1.www.s81c.com';

/**
 * Sets the default location if nothing is returned.
 *
 * @type {Locale}
 * @private
 */
var _localeDefault = {
  lc: 'en',
  cc: 'us'
};

/**
 * Default display name for lang combination.
 *
 * @type {string}
 * @private
 */
var _localeNameDefault = 'United States — English';

/**
 * Locale API endpoint.
 *
 * @type {string}
 * @private
 */
var _endpoint = "".concat(_host, "/common/js/dynamicnav/www/countrylist/jsononly");

/**
 * Configuration for axios.
 *
 * @type {{headers: {'Content-Type': string}}}
 * @private
 */
var _axiosConfig = {
  headers: {
    'Content-Type': 'application/json; charset=utf-8'
  }
};

/**
 * Session Storage key for country list.
 *
 * @type {string}
 * @private
 */
var _sessionListKey = 'dds-countrylist';

/**
 * Two hours in milliseconds to compare session timestamp.
 *
 * @type {number}
 * @private
 */
var _twoHours = 60 * 60 * 2000;

/**
 * The cache for in-flight or resolved requests for the country list, keyed by
 * the initiating locale.
 *
 * @type {object}
 * @private
 */
var _requestsList = {};

/**
 * Retrieves the default locale.
 *
 * @returns {Locale} The default locale.
 */
var _getLocaleDefault = function _getLocaleDefault() {
  return _localeDefault;
};

/**
 * Use the <html> lang attr to determine a return locale object, or "false"
 * when it's not available so the consumer can decide what to do next.
 *
 * @type {(Locale | boolean)}
 * @private
 */
function _getLocaleFromLangAttr() {
  var _root$document;
  if ((_root$document = root.document) !== null && _root$document !== void 0 && (_root$document = _root$document.documentElement) !== null && _root$document !== void 0 && _root$document.lang) {
    var lang = root.document.documentElement.lang.toLowerCase();
    var locale = {};
    if (lang.indexOf('-') === -1) {
      locale.lc = lang;
    } else {
      var codes = lang.split('-');
      locale.cc = codes[1];
      locale.lc = codes[0];
    }
    return locale;
  }
  return false;
}

/**
 * Gets the locale from the cookie and returns it if both 'cc' and 'lc' values
 * are present.
 *
 * @async
 * @returns {Promise<Locale|boolean>} The cookie object if 'cc' and 'lc' values are present, otherwise false.
 */
var _getLocaleFromCookie = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
    var cookie;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          cookie = ipcinfoCookie.get();
          if (!(cookie && cookie.cc && cookie.lc)) {
            _context.next = 5;
            break;
          }
          _context.next = 4;
          return LocaleAPI.getList(cookie);
        case 4:
          return _context.abrupt("return", cookie);
        case 5:
          return _context.abrupt("return", false);
        case 6:
        case "end":
          return _context.stop();
      }
    }, _callee);
  }));
  return function _getLocaleFromCookie() {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Get the locale from the user's browser.
 *
 * @async
 * @returns {Promise<Locale|boolean>} The verified locale or false if not found.
 */
var _getLocaleFromBrowser = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
    var cc, lang, lc, list, verifiedCodes;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) switch (_context2.prev = _context2.next) {
        case 0:
          _context2.prev = 0;
          _context2.next = 3;
          return DDOAPI.getLocation();
        case 3:
          cc = _context2.sent;
          // Language preference from browser can return in either 'en-US' format or
          // 'en' so will need to extract language only.
          lang = root.navigator.language;
          lc = lang.split('-')[0];
          if (!(cc && lc)) {
            _context2.next = 13;
            break;
          }
          _context2.next = 9;
          return LocaleAPI.getList({
            cc: cc,
            lc: lc
          });
        case 9:
          list = _context2.sent;
          verifiedCodes = LocaleAPI.verifyLocale(cc, lc, list); // Set the ipcInfo cookie.
          ipcinfoCookie.set(verifiedCodes);
          return _context2.abrupt("return", verifiedCodes);
        case 13:
          _context2.next = 17;
          break;
        case 15:
          _context2.prev = 15;
          _context2.t0 = _context2["catch"](0);
        case 17:
          return _context2.abrupt("return", false);
        case 18:
        case "end":
          return _context2.stop();
      }
    }, _callee2, null, [[0, 15]]);
  }));
  return function _getLocaleFromBrowser() {
    return _ref2.apply(this, arguments);
  };
}();

/**
 * Return a locale object based on the DDO API, or "false" so the consumer can
 * decide what to do next.
 *
 * @returns {(Locale | boolean)} Locale from the DDO, or "false" if not present.
 * @private
 */
function _getLocaleFromDDO() {
  var _ddoLocal$page;
  var ddoLocal = Object.assign({}, root.digitalData || {});
  if ((_ddoLocal$page = ddoLocal.page) !== null && _ddoLocal$page !== void 0 && (_ddoLocal$page = _ddoLocal$page.pageInfo) !== null && _ddoLocal$page !== void 0 && _ddoLocal$page.language) {
    var _ddoLocal$page2, _ddoLocal$page3;
    var lang = {};

    // if DDO language contains both lc & cc (ie. en-US)
    if ((_ddoLocal$page2 = ddoLocal.page) !== null && _ddoLocal$page2 !== void 0 && (_ddoLocal$page2 = _ddoLocal$page2.pageInfo) !== null && _ddoLocal$page2 !== void 0 && _ddoLocal$page2.language.includes('-') && (_ddoLocal$page3 = ddoLocal.page) !== null && _ddoLocal$page3 !== void 0 && (_ddoLocal$page3 = _ddoLocal$page3.pageInfo) !== null && _ddoLocal$page3 !== void 0 && (_ddoLocal$page3 = _ddoLocal$page3.ibm) !== null && _ddoLocal$page3 !== void 0 && _ddoLocal$page3.country) {
      // Set proper LC for us to use.
      lang.lc = ddoLocal.page.pageInfo.language.substring(0, 2).toLowerCase();
      lang.cc = ddoLocal.page.pageInfo.ibm.country.toLowerCase().trim();

      // If there are multiple countries use just the first one for the CC value
      if (lang.cc.indexOf(',') > -1) lang.cc = lang.cc.substring(0, lang.cc.indexOf(',')).trim();

      // Gb will be uk elsewhere
      if (lang.cc === 'gb') {
        lang.cc = 'uk';
      }

      // Map worldwide (ZZ) pages to US
      if (lang.cc === 'zz') {
        lang.cc = 'us';
      }
    } else {
      // set lc with just the language code
      lang.lc = ddoLocal.page.pageInfo.language.substring(0, 2).toLowerCase();
    }
    return lang;
  }
  return false;
}

/**
 * Locale API class with method of fetching user's locale for ibm.com.
 */
var LocaleAPI = /*#__PURE__*/function () {
  function LocaleAPI() {
    _classCallCheck(this, LocaleAPI);
  }
  _createClass(LocaleAPI, null, [{
    key: "clearCache",
    value:
    /**
     * Clears the cache.
     */
    function clearCache() {
      if (typeof sessionStorage !== 'undefined') {
        Object.keys(_requestsList).forEach(function (key) {
          return delete _requestsList[key];
        });
        for (var i = 0; i < sessionStorage.length; ++i) {
          var key = sessionStorage.key(i);
          if (key.indexOf(_sessionListKey) === 0) {
            sessionStorage.removeItem(key);
          }
        }
      }
    }

    /**
     * Gets the user's locale.
     *
     * Grab the locale from the available information on the page in the following
     * order:
     *
     * 1. DDO
     * 2. HTML lang attribute
     * 3. ipcInfo cookie
     * 4. Browser (navigator.language)
     * 5. Default (us-EN)
     *
     * @returns {Promise<Locale>} Locale object.
     * @example
     * import { LocaleAPI } from '@carbon/ibmdotcom-services';
     *
     * async function getLocale() {
     *   const locale = await LocaleAPI.getLocale();
     *   return locale;
     * }
     */
  }, {
    key: "getLocale",
    value: function () {
      var _getLocale = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
        var localeGetters, _i, _localeGetters, getter, locale;
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) switch (_context3.prev = _context3.next) {
            case 0:
              localeGetters = [_getLocaleFromDDO, _getLocaleFromLangAttr, _getLocaleFromCookie, _getLocaleFromBrowser];
              _i = 0, _localeGetters = localeGetters;
            case 2:
              if (!(_i < _localeGetters.length)) {
                _context3.next = 12;
                break;
              }
              getter = _localeGetters[_i];
              _context3.next = 6;
              return getter();
            case 6:
              locale = _context3.sent;
              if (!locale) {
                _context3.next = 9;
                break;
              }
              return _context3.abrupt("return", locale);
            case 9:
              _i++;
              _context3.next = 2;
              break;
            case 12:
              return _context3.abrupt("return", _getLocaleDefault());
            case 13:
            case "end":
              return _context3.stop();
          }
        }, _callee3);
      }));
      function getLocale() {
        return _getLocale.apply(this, arguments);
      }
      return getLocale;
    }()
    /**
     * Gets the user's locale.
     *
     * @returns {Promise<Locale>} Locale object.
     * @example
     * import { LocaleAPI } from '@carbon/ibmdotcom-services';
     *
     * function async getLocale() {
     *    const locale = await LocaleAPI.getLang();
     * }
     *
     * @deprecated in favor of LocalAPI.getLocale.
     */
  }, {
    key: "getLang",
    value: function () {
      var _getLang = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4() {
        return _regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) switch (_context4.prev = _context4.next) {
            case 0:
              return _context4.abrupt("return", this.getLocale());
            case 1:
            case "end":
              return _context4.stop();
          }
        }, _callee4, this);
      }));
      function getLang() {
        return _getLang.apply(this, arguments);
      }
      return getLang;
    }()
    /**
     * This fetches the language display name based on locale.
     *
     * @param {(Locale | boolean)} locale (optional) If not given, uses LocaleAPI.getLocale logic.
     * @returns {Promise<string>} Display name of locale/language.
     */
  }, {
    key: "getLangDisplay",
    value: function () {
      var _getLangDisplay = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5(locale) {
        var lang, list, countries, location;
        return _regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) switch (_context5.prev = _context5.next) {
            case 0:
              if (!locale) {
                _context5.next = 4;
                break;
              }
              _context5.t0 = locale;
              _context5.next = 7;
              break;
            case 4:
              _context5.next = 6;
              return this.getLocale();
            case 6:
              _context5.t0 = _context5.sent;
            case 7:
              lang = _context5.t0;
              _context5.next = 10;
              return this.getList(lang);
            case 10:
              list = _context5.sent;
              // combines the countryList arrays
              countries = [];
              list.regionList.forEach(function (region) {
                countries = countries.concat(region.countryList);
              });

              // get match for countries with multiple languages
              location = countries.filter(function (country) {
                var htmlLang = country.locale.findIndex(function (loc) {
                  return loc[0] === "".concat(lang.lc, "-").concat(lang.cc);
                });
                if (htmlLang !== -1) {
                  var _country$locale;
                  var localeMatch = country.locale.filter(function (l) {
                    return l.includes("".concat(lang.lc, "-").concat(lang.cc));
                  });
                  (_country$locale = country.locale).splice.apply(_country$locale, [0, country.locale.length].concat(_toConsumableArray(localeMatch)));
                  return country;
                }
              });
              if (!location.length) {
                _context5.next = 18;
                break;
              }
              return _context5.abrupt("return", "".concat(location[0].name, " \u2014 ").concat(location[0].locale[0][1]));
            case 18:
              return _context5.abrupt("return", _localeNameDefault);
            case 19:
            case "end":
              return _context5.stop();
          }
        }, _callee5, this);
      }));
      function getLangDisplay(_x) {
        return _getLangDisplay.apply(this, arguments);
      }
      return getLangDisplay;
    }()
    /**
     * Get the country list of all supported countries and their languages
     * if it is not already stored in session storage.
     *
     * @param {Locale} locale Locale object.
     * @param {string} locale.cc Country code.
     * @param {string} locale.lc Language code.
     * @returns {Promise<any>} Promise object.
     * @example
     * import { LocaleAPI } from '@carbon/ibmdotcom-services';
     *
     * function async getLocale() {
     *    const list = await LocaleAPI.getList({ cc: 'us', lc: 'en' });
     *    return list;
     * }
     */
  }, {
    key: "getList",
    value: function () {
      var _getList = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee6(_ref3) {
        var _this = this;
        var cc, lc;
        return _regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) switch (_context6.prev = _context6.next) {
            case 0:
              cc = _ref3.cc, lc = _ref3.lc;
              return _context6.abrupt("return", new Promise(function (resolve, reject) {
                _this.fetchList(cc, lc, resolve, reject);
              }));
            case 2:
            case "end":
              return _context6.stop();
          }
        }, _callee6);
      }));
      function getList(_x2) {
        return _getList.apply(this, arguments);
      }
      return getList;
    }()
    /**
     * Fetches the list data based on cc/lc combination.
     *
     * @param {string} cc Country code.
     * @param {string} lc Language code.
     * @param {Function} resolve Resolves the Promise.
     * @param {Function} reject Rejects the promise.
     */
  }, {
    key: "fetchList",
    value: function fetchList(cc, lc, resolve, reject) {
      var _this2 = this;
      var key = cc !== 'undefined' ? "".concat(lc, "-").concat(cc) : "".concat(lc);
      var itemKey = "".concat(_sessionListKey, "-").concat(key);
      var sessionList = this.getSessionCache(itemKey);
      if (sessionList) {
        resolve(sessionList);
      } else {
        if (!_requestsList[key]) {
          var url = "".concat(_endpoint, "/").concat(cc !== 'undefined' ? "".concat(cc).concat(lc) : "".concat(lc), "-utf8.json");
          _requestsList[key] = axios.get(url, _axiosConfig).then(function (response) {
            var data = response.data;
            data['timestamp'] = Date.now();
            sessionStorage.setItem("".concat(_sessionListKey, "-").concat(key), JSON.stringify(data));
            return data;
          });
        }
        _requestsList[key].then(resolve, function (error) {
          if (cc === _localeDefault.cc && lc === _localeDefault.lc) {
            reject(error);
          } else {
            _this2.fetchList(_localeDefault.cc, _localeDefault.lc, resolve, reject);
          }
        });
      }
    }

    /**
     * Verify that the cc and lc combo is in the list of supported cc-lc combos.
     *
     * @param {string} cc Country code.
     * @param {string} lc Language code.
     * @param {object} list Country list.
     * @returns {object} Object with lc and cc.
     * @example
     * import { LocaleAPI } from '@carbon/ibmdotcom-services';
     *
     * async function getLocale() {
     *   const locale = await LocaleAPI.verifyLocale(cc, lc, data);
     *   return locale;
     * }
     */
  }, {
    key: "verifyLocale",
    value: function verifyLocale(cc, lc, list) {
      var priorityLC;
      var locale;
      var language = list && list.regionList.forEach(function (region) {
        return region.countryList.forEach(function (country) {
          var code = country.locale[0][0].split('-');
          var languageCode = code[0];
          var countryCode = code[1];
          if (countryCode === cc && languageCode === lc) {
            locale = {
              cc: cc,
              lc: lc
            };
          }
          // save the priority language associated with the user's country code
          else if (countryCode === cc && !priorityLC) {
            priorityLC = languageCode;
          }
        });
      });
      if (!language && priorityLC) {
        locale = {
          cc: cc,
          lc: priorityLC
        };
      }
      return locale;
    }

    /**
     * Retrieves session cache and checks if cache needs to be refreshed
     *
     * @param {string} key Session storage key.
     */
  }, {
    key: "getSessionCache",
    value: function getSessionCache(key) {
      var session = typeof sessionStorage === 'undefined' ? undefined : JSON.parse(sessionStorage.getItem(key));
      if (!session || !session.timestamp) {
        return;
      }
      var currentTime = Date.now(),
        timeDiff = currentTime - session.timestamp;
      if (timeDiff > _twoHours) {
        sessionStorage.removeItem(key);
        return;
      }
      return session;
    }
  }]);
  return LocaleAPI;
}();
export default LocaleAPI;