"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _windowOrGlobal = _interopRequireDefault(require("window-or-global"));
/**
 * Copyright IBM Corp. 2020, 2022
 *
 * This source code is licensed under the Apache-2.0 license found in the
 * LICENSE file in the root directory of this source tree.
 */

/**
 * @constant {boolean} scrollTracker determines whether scroll tracking analytics is enabled
 * @private
 */
var _scrollTracker = process && process.env.SCROLL_TRACKING === 'true' || false;

/**
 * Current NODE_ENV
 *
 * @type {string | string}
 * @private
 */
var _env = process && process.env.NODE_ENV || 'development';

/**
 * Analytics API class with methods for firing analytics events on
 * ibm.com
 */
var AnalyticsAPI = /*#__PURE__*/function () {
  function AnalyticsAPI() {
    (0, _classCallCheck2.default)(this, AnalyticsAPI);
  }
  (0, _createClass2.default)(AnalyticsAPI, null, [{
    key: "registerEvent",
    value:
    /**
     * This method checks that the analytics script has been loaded
     * and fires an event to Coremetrics
     *
     * @param {object} eventData Object with standard IBM metric event properties and values to send to Coremetrics
     * @example
     * import { AnalyticsAPI } from '@carbon/ibmdotcom-services';
     *
     * function fireEvent() {
     *    const eventData = {
     *        type: 'element',
     *        primaryCategory: 'MASTHEAD',
     *        eventName: 'CLICK',
     *        executionPath: 'masthead__profile',
     *        execPathReturnCode: 'none',
     *        targetTitle: 'profile'
     *    }
     *    AnalyticsAPI.registerEvent(eventData);
     * }
     */
    function registerEvent(eventData) {
      if (_windowOrGlobal.default.ibmStats) {
        _windowOrGlobal.default.ibmStats.event(eventData);
      }
    }

    /**
     * Initializes all analytics global tracking init functions
     */
  }, {
    key: "initAll",
    value: function initAll() {
      this.initScrollTracker();
      this.initDynamicTabs();
      this.initModals();
    }

    /**
     *
     * If scroll tracking is enabled, this method will fire an event for every 400px
     * user scrolls down the page. Only the deepest depth will fire the event (e.g if
     * user scrolls back up the page, the event will not be triggered)
     *
     * @example
     * import { AnalyticsAPI } from '@carbon/ibmdotcom-services';
     *
     * function init() {
     *   AnalyticsAPI.initScrollTracker();
     * }
     * @deprecated
     */
  }, {
    key: "initScrollTracker",
    value: function initScrollTracker() {
      if (_scrollTracker) {
        console.warn('Scroll tracker service has been deprecated. Please refer to documentation for IBM DBDM gestures 2.0.');
      }
    }

    /**
     * This instantiates an event listener to trigger an event if the Carbon
     * Tabs component is being interacted with by the user
     *
     * @example
     * import { AnalyticsAPI } from '@carbon/ibmdotcom-services';
     *
     * function init() {
     *   AnalyticsAPI.initDynamicTabs();
     * }
     */
  }, {
    key: "initDynamicTabs",
    value: function initDynamicTabs() {
      var tabSelected = this.triggerTabSelected.bind(this);
      _windowOrGlobal.default.document.addEventListener('tab-selected', function (evt) {
        tabSelected(evt.target.id, evt.detail.item.innerText);
      });
    }

    /**
     * Triggers to CLICK event for the dynamic tabs
     *
     * @param {string} executionPath Target ID
     * @param {string} targetTitle Target innerText
     */
  }, {
    key: "triggerTabSelected",
    value: function triggerTabSelected(executionPath, targetTitle) {
      try {
        this.registerEvent({
          type: 'element',
          primaryCategory: 'WIDGET',
          eventName: 'CLICK',
          eventCategoryGroup: 'TABS DYNAMIC',
          executionPath: executionPath,
          targetTitle: targetTitle
        });
      } catch (err) {
        if (_env !== 'production') {
          console.error('Error triggering tab event:', err);
        }
      }
    }

    /**
     * This instantiates an event listener to trigger an event if the Carbon
     * Modal component is being interacted with by the user
     *
     * @example
     * import { AnalyticsAPI } from '@carbon/ibmdotcom-services';
     *
     * function init() {
     *   AnalyticsAPI.initModals();
     * }
     */
  }, {
    key: "initModals",
    value: function initModals() {
      var modalHide = this.triggerModalHide.bind(this);
      _windowOrGlobal.default.document.addEventListener('modal-hidden', function (evt) {
        modalHide(evt.target.id, evt.detail.launchingElement.innerText);
      });
      var modalShow = this.triggerModalShow.bind(this);
      _windowOrGlobal.default.document.addEventListener('modal-shown', function (evt) {
        modalShow(evt.target.id, evt.detail.launchingElement.innerText);
      });
    }

    /**
     * Triggers the HIDE event for the modal
     *
     * @param {string} executionPath Target ID
     * @param {string} targetTitle Target innerText
     */
  }, {
    key: "triggerModalHide",
    value: function triggerModalHide(executionPath, targetTitle) {
      try {
        this.registerEvent({
          type: 'element',
          primaryCategory: 'WIDGET',
          eventName: 'HIDE',
          eventCategoryGroup: 'SHOWHIDE',
          executionPath: executionPath,
          targetTitle: targetTitle
        });
      } catch (err) {
        if (_env !== 'production') {
          console.error('Error triggering modal hide event:', err);
        }
      }
    }

    /**
     * Triggers the SHOW event for the modal
     *
     * @param {string} executionPath Target ID
     * @param {string} targetTitle Target innerText
     */
  }, {
    key: "triggerModalShow",
    value: function triggerModalShow(executionPath, targetTitle) {
      try {
        this.registerEvent({
          type: 'element',
          primaryCategory: 'WIDGET',
          eventName: 'SHOW',
          eventCategoryGroup: 'SHOWHIDE',
          executionPath: executionPath,
          targetTitle: targetTitle
        });
      } catch (err) {
        if (_env !== 'production') {
          console.error('Error triggering modal show event:', err);
        }
      }
    }

    /**
     * Sends video player metrics data
     *
     * @param {object} data event data from the video player
     * @example
     * import { AnalyticsAPI } from '@carbon/ibmdotcom-services';
     *
     *function init() {
     *    const data = {
     *       playerType: 'kaltura',
     *       title: 'Folgers Coffee',
     *       currentTime: 1,
     *       duration: 60,
     *       playerState: 1,
     *       mediaId: '1_9h94wo6b',
     *    };
     *
     *    AnalyticsAPI.videoPlayerStats(data);
     *}
     */
  }, {
    key: "videoPlayerStats",
    value: function videoPlayerStats(data) {
      var playerState = (data === null || data === void 0 ? void 0 : data.playerState) || '',
        currentTime = Math.floor(data.currentTime),
        duration = Math.floor(data.duration),
        percentWatched = Math.floor(currentTime / duration * 100);

      // Set nicenames for player states for event.
      switch (data.playerState) {
        case 0:
          playerState = 'launched';
          break;
        case 1:
          playerState = 'paused';
          break;
        case 2:
          playerState = 'played';
          break;
        case 3:
          playerState = 'ended';
          break;
        case 99:
          playerState = 'error';
          break;
        default:
          if (typeof playerState === 'number') {
            playerState = '';
          }
      }
      if (currentTime === 0) {
        currentTime = 'start';
        percentWatched = '0';
      }
      if (currentTime >= duration || data.playerState === 3) {
        currentTime = 'end';
        percentWatched = '100';
      }

      // If went to the end of the video, and fired "pause" event, don't fire pause event b/c it's really
      // the end of the video, so just let "end" event fire and tag metrics.
      if (currentTime === 'end' && data.playerState === 1) {
        return;
      }
      var eventData = {
        type: 'video',
        primaryCategory: 'VIDEO',
        eventName: data.title,
        eventCategoryGroup: data.playerType,
        executionPath: data.videoId || data.mediaId,
        execPathReturnCode: playerState,
        eventVidStatus: data.playerState,
        eventVidTimeStamp: currentTime,
        eventVidLength: duration,
        eventVidPlayed: percentWatched + '%'
      };
      if (data !== null && data !== void 0 && data.customMetricsData) {
        Object.keys(data.customMetricsData).forEach(function (customMetricsKey) {
          eventData[customMetricsKey] = data.customMetricsData[customMetricsKey];
        });
      }
      try {
        this.registerEvent(eventData);
      } catch (err) {
        if (_env !== 'production') {
          console.error('Error firing video metrics:', err);
        }
      }
    }
  }]);
  return AnalyticsAPI;
}();
var _default = exports.default = AnalyticsAPI;